@@ -5,7 +5,7 @@ use strict;
 use 5.008003;
 no warnings 'utf8';
 
-our $VERSION = '0.999';
+our $VERSION = '1.016_01';
 
 use Encode                qw( decode );
 use File::Basename        qw( basename );
@@ -14,12 +14,14 @@ use Getopt::Long          qw( GetOptions );
 
 use Encode::Locale   qw( decode_argv );
 use File::HomeDir    qw();
+use File::Which      qw( which );
 use Term::Choose     qw();
 use Term::TablePrint qw( print_table );
 
 use App::DBBrowser::Opt;
 use App::DBBrowser::DB;
-#use App::DBBrowser::Join_Union;  # "require"-d
+#use App::DBBrowser::Join_Union;  # 'require'-d
+#use App::DBBrowser::CreateTable; # 'require'-d
 use App::DBBrowser::Table;
 use App::DBBrowser::Auxil;
 
@@ -37,15 +39,16 @@ sub new {
         lyt_3      => {                      layout => 3,             justify => 0, clear_screen => 1, mouse => 0, undef => '  BACK' },
         lyt_stmt_v => { prompt => 'Choose:', layout => 3,             justify => 0, clear_screen => 0, mouse => 0, undef => '  BACK' },
         lyt_stop   => {                                                             clear_screen => 0, mouse => 0                    },
-        quit      => 'QUIT',
-        back      => 'BACK',
-        _quit     => '  QUIT',
-        _back     => '  BACK',
-        _continue => '  CONTINUE',
-        _confirm  => '  CONFIRM',
-        _reset    => '  RESET',
-        ok        => '- OK -',
-        conf_back => '  <=',
+        quit       => 'QUIT',
+        back       => 'BACK',
+        #confirm    => 'CONFIRM',
+        _quit      => '  QUIT',
+        _back      => '  BACK',
+        _continue  => '  CONTINUE',
+        _confirm   => '  CONFIRM',
+        _reset     => '  RESET',
+        ok         => '- OK -',
+        back_short => '  <=',
         clear_screen      => "\e[H\e[J",
         line_fold         => { Charset=> 'utf8', OutputCharset => '_UNICODE_', Urgent => 'FORCE' },
         config_generic    => 'Generic',
@@ -57,11 +60,12 @@ sub new {
                                " = col", " != col", " <> col", " < col", " > col", " >= col", " <= col",
                                "LIKE %col%", "NOT LIKE %col%",  "LIKE col%", "NOT LIKE col%", "LIKE %col", "NOT LIKE %col" ],
                                # "LIKE col", "NOT LIKE col"
-        hidd_func_pr      => { Epoch_to_Date => 'DATE', Truncate => 'TRUNC', Epoch_to_DateTime => 'DATETIME',
-                               Bit_Length => 'BIT_LENGTH', Char_Length => 'CHAR_LENGTH' },
-        keys_hidd_func_pr => [ qw( Epoch_to_Date Bit_Length Truncate Char_Length Epoch_to_DateTime ) ],
-        csv_opt           => [ qw( allow_loose_escapes allow_loose_quotes allow_whitespace auto_diag
-                                   blank_is_undef binary empty_is_undef eol escape_char quote_char sep_char ) ],
+        lock             => 0,
+        scalar_func_h    => { Epoch_to_Date => 'DATE', Truncate => 'TRUNC', Epoch_to_DateTime => 'DATETIME',
+                              Bit_Length => 'BIT_LENGTH', Char_Length => 'CHAR_LENGTH' },
+        scalar_func_keys => [ qw( Epoch_to_Date Bit_Length Truncate Char_Length Epoch_to_DateTime ) ],
+        csv_opt          => [ qw( allow_loose_escapes allow_loose_quotes allow_whitespace auto_diag
+                                  blank_is_undef binary empty_is_undef eol escape_char quote_char sep_char ) ],
     };
     return bless { info => $info }, $class;
 }
@@ -75,8 +79,14 @@ sub __init {
         print "'db-browser' requires a home directory\n";
         exit;
     }
-    my $my_data = decode( 'locale', File::HomeDir->my_data() );
-    my $app_dir = $my_data ? catdir( $my_data, 'db_browser_conf' ) : catdir( $home, '.db_browser_conf' );
+    my $config_home;
+    if ( which( 'xdg-user-dir' ) ) {
+        $config_home = decode 'locale_fs', File::HomeDir::FreeDesktop->my_config();
+    }
+    else {
+        $config_home = decode 'locale_fs', File::HomeDir->my_data();
+    }
+    my $app_dir = $config_home ? catdir( $config_home, 'db_browser' ) : catdir( $home, '.db_browser' );
     mkdir $app_dir or die $! if ! -d $app_dir;
     $self->{info}{home_dir}      = $home;
     $self->{info}{app_dir}       = $app_dir;
@@ -86,8 +96,8 @@ sub __init {
 
     if ( ! eval {
         my $obj_opt = App::DBBrowser::Opt->new( $self->{info}, {}, {} );
-        $self->{opt}    = $obj_opt->read_config_files();
-        $self->{db_opt} = $obj_opt->read_db_config_files();
+        $self->{opt}    = $obj_opt->__read_config_files();
+        $self->{db_opt} = $obj_opt->__read_db_config_files();
         my $help;
         GetOptions (
             'h|?|help' => \$help,
@@ -100,7 +110,7 @@ sub __init {
                     $self->{info}{$key}{mouse} = $self->{opt}{table}{mouse};
                 }
             }
-            ( $self->{opt}, $self->{db_opt} ) = $obj_opt->set_options();
+            ( $self->{opt}, $self->{db_opt} ) = $obj_opt->__set_options();
             if ( defined $self->{opt}{table}{mouse} ) {
                 for my $key ( keys %{$self->{info}} ) {
                     next if $key !~ /^lyt_/;
@@ -131,25 +141,34 @@ sub __init {
 sub __prepare_connect_parameter {
     my ( $self, $db ) = @_;
     my $obj_db = App::DBBrowser::DB->new( $self->{info}, $self->{opt} );
-    my $connect_attr = $obj_db->connect_attributes();
-    my $login_data = $obj_db->login_data();
+    my $env_variables = $obj_db->environment_variables();
+    my $read_arg      = $obj_db->read_arguments();
+    my $chosen_arg    = $obj_db->choose_arguments();
     my $connect_parameter = {
-        attributes => {},
-        login_data => {},
-        login_mode => {},
-        dir_sqlite => [],
+        use_env_var => {},
+        required    => {},
+        keep_secret => {},
+        read_arg    => {},
+        chosen_arg  => {},
+        dir_sqlite  => [],
     };
     my $db_plugin = $self->{info}{db_plugin};
     my $section = $db ? $db_plugin . '_' . $db : $db_plugin;
+    for my $env_var ( @$env_variables ) {
+        if ( defined $db && ! defined $self->{db_opt}{$section}{$env_var} ) {
+            $section = $db_plugin;
+        }
+        $connect_parameter->{use_env_var}{$env_var} = $self->{db_opt}{$section}{$env_var};
+    }
     for my $option ( keys %{$self->{db_opt}{$db_plugin}} ) {
         if ( defined $db && ! defined $self->{db_opt}{$section}{$option} ) {
             $section = $db_plugin;
         }
         if ( defined $self->{info}{driver_prefix} && $option =~ /^\Q$self->{info}{driver_prefix}\E/ ) {
-            $connect_parameter->{attributes}{$option} = $self->{db_opt}{$section}{$option};
+            $connect_parameter->{chosen_arg}{$option} = $self->{db_opt}{$section}{$option};
         }
     }
-    for my $attr ( @$connect_attr ) {
+    for my $attr ( @$chosen_arg ) {
         my $name = $attr->{name};
         if ( defined $db && ! defined $self->{db_opt}{$section}{$name} ) {
             $section = $db_plugin;
@@ -157,31 +176,33 @@ sub __prepare_connect_parameter {
         if ( ! defined $self->{db_opt}{$section}{$name} ) {
             $self->{db_opt}{$section}{$name} = $attr->{avail_values}[$attr->{default_index}];
         }
-        $connect_parameter->{attributes}{$name} = $self->{db_opt}{$section}{$name};
+        $connect_parameter->{chosen_arg}{$name} = $self->{db_opt}{$section}{$name};
     }
-    for my $item ( @$login_data ) {
+    for my $item ( @$read_arg ) {
         my $name = $item->{name};
-        my $login_mode_key = 'login_mode_' . $name;
+        my $required_field = 'field_' . $name;
         $connect_parameter->{keep_secret}{$name} = $item->{keep_secret};
-        if ( defined $db && ! defined $self->{db_opt}{$section}{$login_mode_key} ) {
+        if ( defined $db && ! defined $self->{db_opt}{$section}{$required_field} ) {
             $section = $db_plugin;
         }
-        if ( ! defined $self->{db_opt}{$section}{$login_mode_key} ) {
-            $self->{db_opt}{$section}{$login_mode_key} = 0;
+        if ( ! defined $self->{db_opt}{$section}{$required_field} ) {
+            $self->{db_opt}{$section}{$required_field} = 1; # All fields required by default
         }
-        $connect_parameter->{login_mode}{$name} = $self->{db_opt}{$section}{$login_mode_key};
-        if ( ! $self->{info}{error} ) {
+        $connect_parameter->{required}{$name} = $self->{db_opt}{$section}{$required_field};
+        if ( ! $self->{info}{login_error} ) {
             if ( defined $db && ! defined $self->{db_opt}{$section}{$name} ) {
                 $section = $db_plugin;
             }
-            $connect_parameter->{login_data}{$name} = $self->{db_opt}{$section}{$name};
+            $connect_parameter->{read_arg}{$name} = $self->{db_opt}{$section}{$name};
         }
     }
-    if ( ! defined $self->{db_opt}{$db_plugin}{directories_sqlite} ) {
+    if ( $self->{info}{db_driver} eq 'SQLite' && ! defined $self->{db_opt}{$db_plugin}{directories_sqlite} ) {
         $self->{db_opt}{$db_plugin}{directories_sqlite} = [ $self->{info}{home_dir} ];
     }
     $connect_parameter->{dir_sqlite} = $self->{db_opt}{$db_plugin}{directories_sqlite};
-    delete $self->{info}{error} if exists $self->{info}{error};
+    if ( exists $self->{info}{login_error} ) {
+        delete $self->{info}{login_error};
+    }
     return $connect_parameter;
 }
 
@@ -221,9 +242,6 @@ sub run {
             next DB_PLUGIN;
         }
         my $db_driver = $self->{info}{db_driver};
-        ###
-        $self->{opt}{G}{sqlite_directories} = $self->{db_opt}{$db_plugin}{directories_sqlite};                              ### 1.1
-        $self->{opt}{G}{sqlite_directories} = [ $self->{info}{home_dir} ] if ! defined $self->{opt}{G}{sqlite_directories}; ### 1.1
 
         # DATABASES
 
@@ -243,7 +261,7 @@ sub run {
             1 }
         ) {
             $auxil->__print_error_message( $@, 'Available databases' );
-            $self->{info}{error} = 1;
+            $self->{info}{login_error} = 1;
             next DB_PLUGIN;
         }
         if ( ! @$databases ) {
@@ -306,7 +324,7 @@ sub run {
             ) {
                 $auxil->__print_error_message( $@, 'Get database handle' );
                 # remove database from @databases
-                $self->{info}{error} = 1;
+                $self->{info}{login_error} = 1;
                 next DATABASE;
             }
 
@@ -333,7 +351,11 @@ sub run {
             SCHEMA: while ( 1 ) {
 
                 my $schema;
-                if ( @schemas <= 1 ) {
+                if ( $self->{info}{redo_schema} ) {
+                    $schema = $self->{info}{redo_schema};
+                    delete $self->{info}{redo_schema};
+                }
+                elsif ( @schemas <= 1 ) {
                     $schema = $schemas[0];
                     $auto_one++ if $auto_one == 2
                 }
@@ -379,18 +401,19 @@ sub run {
                     $auxil->__print_error_message( $@, 'Get table names' );
                     next DATABASE;
                 }
-                my $old_idx_tbl = 0;
+                my $old_idx_tbl = 1;
 
                 TABLE: while ( 1 ) {
 
-                    my ( $join, $union, $db_setting ) = ( '  Join', '  Union', '  Database settings' );
-                    my $choices_table = [ undef, @tables, $join, $union, $db_setting ];
-                    my $back   = $auto_one == 3 ? $self->{info}{_quit} : $self->{info}{_back};
-                    my $prompt = 'DB: "'. basename( $db ) . ( @schemas > 1 ? '.' . $schema : '' ) . '"';
+                    my $db_string = 'DB: "'. basename( $db ) . ( @schemas > 1 ? '.' . $schema : '' ) . '"';
+                    my ( $join, $union, $new, $db_setting ) = ( '  Join', '  Union', '  New', '  Database settings' );
+                    my $hidden = $db_string;
+                    my $choices_table = [ $hidden, undef, @tables, $join, $union, $db_setting ];
+                    my $back = $auto_one == 3 ? $self->{info}{_quit} : $self->{info}{_back};
                     # Choose
                     my $idx_tbl = $lyt_3->choose(
                         $choices_table,
-                        { prompt => $prompt, index => 1, default => $old_idx_tbl, undef => $back }
+                        { prompt => '', index => 1, default => $old_idx_tbl, undef => $back }
                     );
                     my $table = $choices_table->[$idx_tbl] if defined $idx_tbl;
                     if ( ! defined $table ) {
@@ -401,19 +424,18 @@ sub run {
                     }
                     if ( $self->{opt}{G}{menus_db_memory} ) {
                         if ( $old_idx_tbl == $idx_tbl ) {
-                            $old_idx_tbl = 0;
+                            $old_idx_tbl = 1;
                             next TABLE;
                         }
                         else {
                             $old_idx_tbl = $idx_tbl;
                         }
                     }
-                    my $multi_table;
                     if ( $table eq $db_setting ) {
                         my $new_db_settings;
                         if ( ! eval {
                             my $obj_opt = App::DBBrowser::Opt->new( $self->{info}, $self->{opt}, $self->{db_opt} );
-                            $new_db_settings = $obj_opt->database_setting( $db );
+                            $new_db_settings = $obj_opt->__database_setting( $db );
                             1 }
                         ) {
                             $auxil->__print_error_message( $@, 'Database settings' );
@@ -422,42 +444,132 @@ sub run {
                         next SCHEMA if $new_db_settings;
                         next TABLE;
                     }
-                    elsif ( $table eq $join ) {
+                    my $sql = {
+                        print => {
+                            db     => $db,
+                            schema => $schema,
+                        }
+                    };
+                    if ( $table eq $join ) {
+                        my $ok;
                         if ( ! eval {
                             require App::DBBrowser::Join_Union;
                             my $obj_ju = App::DBBrowser::Join_Union->new( $self->{info}, $self->{opt} );
-                            $multi_table = $obj_ju->__join_tables( $dbh, $db, $schema, $data );
+                            $ok = $obj_ju->__join_tables( $sql, $dbh, $data );
                             $table = 'joined_tables';
+                            $sql->{print}{table} = $table;
+                            $sql->{from_stmt_type} = 'join';
                             1 }
                         ) {
                             $auxil->__print_error_message( $@, 'Join tables' );
                             next TABLE;
                         }
-                        next TABLE if ! defined $multi_table;
+                        next TABLE if ! $ok;
                     }
                     elsif ( $table eq $union ) {
+                        my $ok;
                         if ( ! eval {
                             require App::DBBrowser::Join_Union;
                             my $obj_ju = App::DBBrowser::Join_Union->new( $self->{info}, $self->{opt} );
-                            $multi_table = $obj_ju->__union_tables( $dbh, $db, $schema, $data );
-                            if ( $obj_ju->{union_all} ) {
+                            $ok = $obj_ju->__union_tables( $sql, $dbh, $data );
+                            if ( $sql->{union_all} ) {
                                 $table = 'union_all_tables';
                             }
                             else {
                                 $table = 'union_selected_tables';
                             }
+                            delete $sql->{union_all};
+                            $sql->{print}{table} = $table;
+                            $sql->{from_stmt_type} = 'union';
                             1 }
                         ) {
                             $auxil->__print_error_message( $@, 'Union tables' );
                             next TABLE;
                         }
-                        next TABLE if ! defined $multi_table;
+                        next TABLE if ! $ok;
+                    }
+                    elsif ( $table eq $hidden ) {
+                        ####
+                        my $pi_module = "App::DBBrowser::DB::$db_plugin";                                                   # 1.4
+                        my $np = $pi_module->new( $self->{opt} );                                                           # 1.4
+                        if ( ! $np->can( 'create_table' ) || ! $np->can( 'drop_table' ) ) {                                 # 1.4
+                            my $message = qq{"create_table" or "drop_table" not available.\n};                              # 1.4
+                            $auxil->__print_error_message( $message, "App::DBBrowser::DB::$db_plugin" );                    # 1.4
+                            next TABLE;                                                                                     # 1.4
+                        }                                                                                                   # 1.4
+                        ####
+                        $sql->{print}{chosen_cols} = []; #
+                        $sql->{quote}{chosen_cols} = []; #
+                        my $old_idx_hdn = 0;
+
+                        HIDDEN: while ( 1 ) {
+                            my ( $create_table, $drop_table ) = ( '- CREATE table', '- DROP   table' );
+                            my $choices_hidden = [ undef, $create_table, $drop_table ];
+                            # Choose
+                            my $idx_hdn = $lyt_3->choose(
+                                $choices_hidden,
+                                { prompt => $db_string, index => 1, default => $old_idx_hdn, undef => $self->{info}{_back} }
+                            );
+                            my $choice = $choices_hidden->[$idx_hdn] if defined $idx_hdn;
+                            if ( ! defined $choice ) {
+                                next TABLE;
+                            }
+                            if ( $self->{opt}{G}{menus_db_memory} ) {
+                                if ( $old_idx_hdn == $idx_hdn ) {
+                                    $old_idx_hdn = 0;
+                                    next HIDDEN;
+                                }
+                                else {
+                                    $old_idx_hdn = $idx_hdn;
+                                }
+                            }
+                            if ( $choice eq $create_table ) {
+                                if ( ! eval {
+                                    require App::DBBrowser::CreateTable;
+                                    my $obj_ct = App::DBBrowser::CreateTable->new( $self->{info}, $self->{opt} );
+                                    $table = $obj_ct->__create_new_table( $sql, $dbh );
+                                    1 }
+                                ) {
+                                    $auxil->__print_error_message( $@, 'Create table' );
+                                    next HIDDEN;
+                                }
+                                next HIDDEN if ! $table;
+                                $self->{info}{redo_schema} = $schema;
+                                next SCHEMA;
+                            }
+                            elsif ( $choice eq $drop_table ) {
+                                my $ok;
+                                if ( ! eval {
+                                    require App::DBBrowser::CreateTable;
+                                    my $obj_ct = App::DBBrowser::CreateTable->new( $self->{info}, $self->{opt} );
+                                    $ok = $obj_ct->__delete_table( $sql, $dbh );
+                                    1 }
+                                ) {
+                                    $auxil->__print_error_message( $@, 'Drop table' );
+                                    next HIDDEN;
+                                }
+                                next HIDDEN if ! $ok;
+                                $self->{info}{redo_schema} = $schema;
+                                next SCHEMA;
+                            }
+                        }
                     }
                     else {
                         $table =~ s/^[-\ ]\s//;
+                        my $qt_table = $dbh->quote_identifier( undef, $schema, $table );
+                        my $sth = $dbh->prepare( "SELECT * FROM " . $qt_table . " LIMIT 0" );
+                        $sth->execute();
+                        for my $col ( @{$sth->{NAME}} ) {
+                            $sql->{quote}{columns}{$col} = $dbh->quote_identifier( $col );
+                            push @{$sql->{print}{columns}}, $col;
+                        }
+                        $sql->{quote}{col_stmt} = "*";
+                        $sql->{quote}{table} = $qt_table;
+                        $sql->{print}{table} = $table;
+                        $sql->{from_stmt_type} = 'single';
                     }
                     #if ( ! eval {
-                         $self->__browse_the_table( $dbh, $db, $schema, $table, $multi_table );
+                         $self->__browse_the_table( $dbh, $sql );
                     #    1 }
                     #) {
                     #    $auxil->__print_error_message( $@, 'Browse table' );
@@ -472,33 +584,14 @@ sub run {
 
 
 sub __browse_the_table {
-    my ( $self, $dbh, $db, $schema, $table, $multi_table ) = @_;
-    my $auxil     = App::DBBrowser::Auxil->new( $self->{info} );
+    my ( $self, $dbh, $sql ) = @_;
+    my $auxil = App::DBBrowser::Auxil->new( $self->{info} );
     my $db_plugin = $self->{info}{db_plugin};
-    my $qt_columns = {};
-    my $pr_columns = [];
-    my $sql        = {};
+    my $db = $sql->{print}{db};
     $sql->{strg_keys} = [ qw( distinct_stmt set_stmt where_stmt group_by_stmt having_stmt order_by_stmt limit_stmt ) ];
     $sql->{list_keys} = [ qw( chosen_cols set_args aggr_cols where_args group_by_cols having_args insert_into_args ) ];
     $auxil->__reset_sql( $sql );
     $self->{info}{lock} = $self->{opt}{G}{lock_stmt};
-    my $select_from_stmt = '';
-    if ( $multi_table ) {
-        $select_from_stmt = $multi_table->{quote}{stmt};
-        for my $col ( @{$multi_table->{pr_columns}} ) {
-            $qt_columns->{$col} = $multi_table->{qt_columns}{$col};
-            push @$pr_columns, $col;
-        }
-    }
-    else {
-        $select_from_stmt = "SELECT * FROM " . $dbh->quote_identifier( undef, $schema, $table );
-        my $sth = $dbh->prepare( $select_from_stmt . " LIMIT 0" );
-        $sth->execute();
-        for my $col ( @{$sth->{NAME}} ) {
-            $qt_columns->{$col} = $dbh->quote_identifier( $col );
-            push @$pr_columns, $col;
-        }
-    }
     my $obj_table = App::DBBrowser::Table->new( $self->{info}, $self->{opt} );
     $self->{opt}{table}{binary_filter} = $self->{db_opt}{$db_plugin . '_' . $db}{binary_filter};
     if ( ! defined $self->{opt}{table}{binary_filter} ) {
@@ -508,24 +601,25 @@ sub __browse_the_table {
     PRINT_TABLE: while ( 1 ) {
         my $all_arrayref;
         if ( ! eval {
-            ( $all_arrayref, $sql ) = $obj_table->__on_table( $sql, $dbh, $table, $select_from_stmt, $qt_columns, $pr_columns );
+            ( $all_arrayref, $sql ) = $obj_table->__on_table( $sql, $dbh );
             1 }
         ) {
             $auxil->__print_error_message( $@, 'Print table' );
             next PRINT_TABLE;
         }
         last PRINT_TABLE if ! defined $all_arrayref;
-        delete @{$self->{info}}{qw(width_head width_cols not_a_number)};
         print_table( $all_arrayref, $self->{opt}{table} );
         if ( defined $self->{info}{backup_max_rows} ) {
             $self->{opt}{table}{max_rows} = delete $self->{info}{backup_max_rows};
         }
     }
+    if ( $db_plugin eq 'Debug' ) {
+        my $obj_db = App::DBBrowser::DB->new( $self->{info}, $self->{opt} );
+        $obj_db->debug( $dbh, $self->{info}, $self->{opt}, $self->{db_opt} );
+    }
 }
 
 
-
-
 1;
 
 
@@ -541,7 +635,7 @@ App::DBBrowser - Browse SQLite/MySQL/PostgreSQL databases and their tables inter
 
 =head1 VERSION
 
-Version 0.999
+Version 1.016_01
 
 =head1 DESCRIPTION
 
