@@ -6,7 +6,7 @@ use strict;
 use 5.008003;
 no warnings 'utf8';
 
-our $VERSION = '0.999';
+our $VERSION = '1.016_01';
 
 
 
@@ -16,7 +16,7 @@ App::DBBrowser::DB - Database plugin documentation.
 
 =head1 VERSION
 
-Version 0.999
+Version 1.016_01
 
 =head1 DESCRIPTION
 
@@ -30,26 +30,13 @@ selecting I<DB> and then I<DB Plugins>.
 
 A suitable database plugin provides the methods named in this documentation.
 
-Column names passed as arguments are already quoted with the C<DBI> C<quote_identifier> method.
+Column names passed as arguments to plugin methods are already quoted with the C<DBI> C<quote_identifier> method.
 
 =head1 PLUGIN API VERSION
 
-This documentation describes the plugin API version C<1.3>.
+This documentation describes the plugin API version C<1.5>.
 
-New in C<1.3>:
-
-- method C<column_names_and_types> is not optional (required in union and join ).
-
-New in C<1.2>:
-
-- the former object attribute "metadata" is now called "add_metadata".
-
-- the SQLite directories are passed directly to the C<available_databases> method (see C<$connect_parameter>) instead to
-the constructor C<new>.
-
-Supported plugin API versions: C<1.3>, C<1.2>, C<1.1>.
-
-Support for the version C<1.1> and C<1.2> will be removed soon.
+Supported plugin API versions: C<1.4> and C<1.5>.
 
 =head1 METHODS
 
@@ -96,12 +83,10 @@ sub new {
         db_cache_file       => $info->{db_cache_file},
         sqlite_search       => $info->{sqlite_search},
         clear_screen        => $info->{clear_screen},
-        metadata            => $opt->{G}{metadata},           ### 1.1
         add_metadata        => $opt->{G}{metadata},
-        directories_sqlite  => $opt->{G}{sqlite_directories}, ### 1.1
     } );
 
-    my $minimum_pav = 1.1;
+    my $minimum_pav = 1.4;
 
     my $pav;
     $pav = $plugin->plugin_api_version() if $plugin->can( 'plugin_api_version' );
@@ -114,6 +99,103 @@ sub new {
     bless { Plugin => $plugin }, $class;
 }
 
+sub message_method_undef_return {
+    my ( $self, $method ) = @_;
+    return sprintf '%s method %s: no return value', ref $self->{Plugin}, $method;
+}
+
+sub debug {
+    my ( $self, $dbh, $info, $opt, $db_opt ) = @_;
+    return if ! $self->{Plugin}->can( 'debug' );
+    $self->{Plugin}->debug( $dbh, $info, $opt, $db_opt );
+}
+
+
+
+
+=head2 create_table
+
+=over
+
+=item Arguments
+
+none
+
+=item return
+
+The primary-key-autoincrement statement.
+
+=back
+
+Example for the database driver C<Pg>:
+
+    sub primary_key_auto {
+        return "SERIAL PRIMARY KEY";
+    }
+
+=cut
+
+sub primary_key_auto {
+    my ( $self ) = @_;
+    return if ! $self->{Plugin}->can( 'primary_key_auto' );
+    return $self->{Plugin}->primary_key_auto();
+}
+
+
+
+=head2 create_table
+
+=over
+
+=item Arguments
+
+Database handle, quoted table name ( "schema"."table" ) and a reference to an array of arrays:
+    [ [ col name, col datatype ],
+      [ col name, col datatype ],
+      ...
+    ]
+The column name is already quoted.
+
+=item return
+
+nothing
+
+=back
+
+Creates a new table.
+
+=cut
+
+sub create_table {
+    my ( $self, $dbh, $table, $col_type ) = @_; #, $add_primay_key
+    my $ok = $self->{Plugin}->create_table( $dbh, $table, $col_type ); # , $add_primay_key
+    return $ok;
+}
+
+
+=head2 drop_table
+
+=over
+
+=item Arguments
+
+Database handle and a quoted table name ( "schema"."table" ).
+
+=item return
+
+nothing
+
+=back
+
+Drops the table.
+
+=cut
+
+sub drop_table {
+    my ( $self, $dbh, $table ) = @_;
+    my $ok = $self->{Plugin}->drop_table( $dbh, $table );
+    return $ok;
+}
 
 
 =head2 plugin_api_version
@@ -161,6 +243,7 @@ The name of the C<DBI> database driver used by the plugin.
 sub db_driver {
     my ( $self ) = @_;
     my $db_driver = $self->{Plugin}->db_driver();
+    die $self->message_method_undef_return( 'db_driver' ) if ! defined $db_driver;
     return $db_driver;
 }
 
@@ -200,7 +283,7 @@ sub driver_prefix {
 
 
 
-=head2 login_data
+=head2 read_arguments
 
 =over
 
@@ -210,22 +293,23 @@ none
 
 =item return
 
-A reference to an array of hashes. The hashes have three key-value pairs:
+A reference to an array of hashes. The hashes have two or three key-value pairs:
 
     { name => 'string', prompt => 'string', keep_secret => true/false }
 
-C<name> holds the name of the login data for example like "user" or "host".
+C<name> holds the field name for example like "user" or "host".
 
-The value of C<prompt> is used as the prompt string, when the user is asked for the login data.
+The value of C<prompt> is used as the prompt string, when the user is asked for the data. The C<prompt> entry is
+optional. If C<prompt> doesn't exist, the value of C<name> is used instead.
 
-If C<keep_secret> is true, the user input should not be echoed to the terminal. Also the login data is not stored in the
+If C<keep_secret> is true, the user input should not be echoed to the terminal. Also the data is not stored in the
 plugin configuration file if C<keep_secret> is true.
 
 =back
 
-An example C<login_data> method:
+An example C<read_arguments> method:
 
-    sub login_data {
+    sub read_arguments {
         my ( $self ) = @_;
         return [
             { name => 'host', prompt => "Host",     keep_secret => 0 },
@@ -235,23 +319,27 @@ An example C<login_data> method:
         ];
     }
 
-The information returned by method C<login_data> is used to build the entries of the C<db-browser> options I<DB Login Mode> and
-I<DB Login Data>.
+The information returned by the method C<read_arguments> is used to build the entries of the C<db-browser> options
+I<Fields> and I<Login Data>.
+
+    read_arguments()  =>  option "Fields"      =>  $connect_parameter->{required}
+                          option "Login Data"  =>  $connect_parameter->{read_arg}
+                                               =>  $connect_parameter->{keep_secret}
+
 
 =cut
 
-sub login_data {
+sub read_arguments {
     my ( $self ) = @_;
-    return [] if ! $self->{Plugin}->can( 'login_data' );
-    my $login_data = $self->{Plugin}->login_data();
-    return [] if ! defined $login_data;
-    return $login_data;
+    return [] if ! $self->{Plugin}->can( 'read_arguments' );
+    my $data = $self->{Plugin}->read_arguments();
+    return [] if ! defined $data;
+    return $data;
 }
 
 
 
-
-=head2 connect_attributes
+=head2 environment_variables
 
 =over
 
@@ -261,21 +349,61 @@ none
 
 =item return
 
-A reference to an array of hashes. The hashes have three key-value pairs:
+A reference to an array of environment variables.
 
-    { name => 'string', default_index => index, avail_values => [ value_1, value_2, value_3, ... ] }
+=back
+
+An example C<environment_variables> method:
+
+    sub environment_variables {
+        my ( $self ) = @_;
+        return [ qw( DBI_DSN DBI_HOST DBI_PORT DBI_USER DBI_PASS ) ];
+    }
+
+See the C<db-browser> option I<ENV Variables>.
+
+    environment_variables()  =>  option "ENV Variables"  =>  $connect_parameter->{use_env_var}
+
+=cut
+
+sub environment_variables {
+    my ( $self ) = @_;
+    return [] if ! $self->{Plugin}->can( 'environment_variables' );
+    my $env_variables = $self->{Plugin}->environment_variables();
+    return [] if ! defined $env_variables;
+    return $env_variables;
+}
+
+
+
+=head2 choose_arguments
+
+=over
+
+=item Arguments
+
+none
+
+=item return
+
+A reference to an array of hashes. The hashes have three or four key-value pairs:
+
+    { name => 'string', prompt => 'string', default_index => index, avail_values => [ value_1, value_2, value_3, ... ] }
 
 The value of C<name> is the name of the database connection attribute.
 
-C<avail_values> holds the available values for that connection attribute as an array reference.
+The value of C<prompt> is used as the prompt string. The C<prompt> entry is optional. If C<prompt> doesn't exist, the
+value of C<name> is used instead.
 
-The C<avail_values> array entry of the index position C<default_index> is used as default value.
+C<avail_values> holds the available values for that attribute as an array reference.
+
+The C<avail_values> array entry of the index position C<default_index> is used as the default value.
 
 =back
 
 Example form the plugin C<App::DBBrowser::DB::SQLite>:
 
-    sub connect_attributes {
+    sub choose_arguments {
         my ( $self ) = @_;
         return [
             { name => 'sqlite_unicode',             default_index => 1, avail_values => [ 0, 1 ] },
@@ -283,12 +411,16 @@ Example form the plugin C<App::DBBrowser::DB::SQLite>:
         ];
     }
 
+See the C<db-browser> option I<DB Options>.
+
+    choose_arguments()  =>  option "DB Options"  =>  $connect_parameter->{chosen_arg}
+
 =cut
 
-sub connect_attributes {
+sub choose_arguments {
     my ( $self ) = @_;
-    return [] if ! $self->{Plugin}->can( 'connect_attributes' );
-    my $connect_attributes = $self->{Plugin}->connect_attributes();
+    return [] if ! $self->{Plugin}->can( 'choose_arguments' );
+    my $connect_attributes = $self->{Plugin}->choose_arguments();
     return [] if ! defined $connect_attributes;
     return $connect_attributes;
 }
@@ -309,7 +441,7 @@ passed to C<get_db_handle> as the second argument. See L</get_db_handle> for mor
 If the object attribute I<add_metadata> is true, C<available_databases> returns the "user-databases" as an
 array-reference and the "system-databases" (if any) as an array-reference.
 
-If the object attribute I<add_metadata> is not true, C<available_databases> returns only the "user-databases" as an
+If I<add_metadata> is not true, C<available_databases> returns only the "user-databases" as an
 array-reference.
 
 =back
@@ -335,17 +467,22 @@ The database name and a reference to a hash of hashes.
 The hash of hashes provides the settings gathered from the option I<Database settings>.
 
     $connect_parameter = {
-        attributes => {
-            connect_attribute => chosen value,
-            connect_attribute => chosen value,
+        use_env_var => {
+            env_var => true or false,
+            env_var => true or false,
             ...
         },
-        login_mode => {         # ask   use environment variable   don't ask
-            name => 0, 1 or 2,  # 0     1                          2
-            name => 0, 1 or 2,  # 0     1                          2
+        chosen_arg => {
+            attribute => chosen value,
+            attribute => chosen value,
             ...
         },
-        login_data => {
+        required => {
+            name => true or false,
+            name => true or false,
+            ...
+        },
+        read_arg => {
             name => user input,
             name => user input,
             ...
@@ -361,26 +498,28 @@ The hash of hashes provides the settings gathered from the option I<Database set
         ]
     };
 
-If I<login_mode> is set to C<1>, the name of the used environment variable is the uppercase of "name" prefixed with
-"DBI_".
-
 For example for the plugin C<mysql> the hash of hashes held by C<$connect_parameter> could look like this:
 
     $connect_parameter = {
-        login_data => {
+        use_env_var => {
+            DBI_HOST => 1,
+            DBI_USER => 0,
+            DBI_PASS => 0,
+        },
+        read_arg => {
             host => undef,
             pass => undef,
             user => 'db_user_name',
             port => undef
         },
-        attributes => {
+        chosen_arg => {
             mysql_enable_utf8 => 1
         },
-        login_mode => {
-            port => 2,      # don't ask
-            user => 1,      # use the environment variable DBI_USER if available
-            pass => 1,      # use the environment variable DBI_PASS if available
-            host => 2       # don't ask
+        required => {
+            port => 0,
+            user => 1,
+            pass => 1,
+            host => 1
         },
         keep_secret => {
             port => 0,
@@ -401,6 +540,7 @@ Database handle.
 sub get_db_handle {
     my ( $self, $db, $connect_parameter ) = @_;
     my $dbh = $self->{Plugin}->get_db_handle( $db, $connect_parameter );
+    die $self->message_method_undef_return( 'get_db_handle' ) if ! defined $dbh;
     return $dbh;
 }
 
@@ -416,10 +556,10 @@ The database handle and the database name.
 
 =item return
 
-If the object attribute I<add_metadata> is true, C<get_schema_names> returns the "user-schemas" as an array-reference
+If I<add_metadata> is true, C<get_schema_names> returns the "user-schemas" as an array-reference
 and the "system-schemas" (if any) as an array-reference.
 
-If the object attribute I<add_metadata> is not true, C<get_schema_names> returns only the "user-schemas" as an
+If I<add_metadata> is not true, C<get_schema_names> returns only the "user-schemas" as an
 array-reference.
 
 =back
@@ -445,10 +585,10 @@ The database handle and the schema name.
 
 =item return
 
-If the object attribute I<add_metadata> is true, C<get_table_names> returns the "user-tables" as an array-reference and
+If I<add_metadata> is true, C<get_table_names> returns the "user-tables" as an array-reference and
 the "system-tables" (if any) as an array-reference.
 
-If the object attribute I<add_metadata> is not true, C<get_table_names> returns only the "user-tables" as
+If I<add_metadata> is not true, C<get_table_names> returns only the "user-tables" as
 an array-reference.
 
 =back
@@ -494,6 +634,8 @@ Two hash references - one for the column names and one for the column types:
 sub column_names_and_types {
     my ( $self, $dbh, $db, $schema, $tables ) = @_;
     my ( $col_names, $col_types ) = $self->{Plugin}->column_names_and_types( $dbh, $db, $schema, $tables );
+    die $self->message_method_undef_return( 'column_names_and_types' ) if ! defined $col_names;
+    $col_types = {} if ! defined $col_types;
     for my $table ( keys %$col_types ) {
         for ( @{$col_types->{$table}} ) {
             s/integer/int/i;
@@ -586,6 +728,7 @@ Example form the plugin C<App::DBBrowser::DB::mysql>:
 sub sql_regexp {
     my ( $self, $quote_col, $do_not_match_regexp, $case_sensitive ) = @_;
     my $sql_regexp = $self->{Plugin}->sql_regexp( $quote_col, $do_not_match_regexp, $case_sensitive );
+    die $self->message_method_undef_return( 'sql_regexp' ) if ! defined $sql_regexp;
     $sql_regexp = ' ' . $sql_regexp if $sql_regexp !~ /^\ /;
     return $sql_regexp;
 }
@@ -618,6 +761,7 @@ Example form the plugin C<App::DBBrowser::DB::Pg>:
 sub concatenate {
     my ( $self, $arg ) = @_;
     my $concatenated = $self->{Plugin}->concatenate( $arg );
+    die $self->message_method_undef_return( 'concatenate' ) if ! defined $concatenated;
     return $concatenated;
 }
 
@@ -654,6 +798,7 @@ Example form the plugin C<App::DBBrowser::DB::mysql>:
 sub epoch_to_datetime {
     my ( $self, $quote_col, $interval ) = @_;
     my $quote_f = $self->{Plugin}->epoch_to_datetime( $quote_col, $interval );
+    die $self->message_method_undef_return( 'epoch_to_datetime' ) if ! defined $quote_f;
     return $quote_f;
 }
 
@@ -687,6 +832,7 @@ Example form the plugin C<App::DBBrowser::DB::mysql>:
 sub epoch_to_date {
     my ( $self, $quote_col, $interval ) = @_;
     my $quote_f = $self->{Plugin}->epoch_to_date( $quote_col, $interval );
+    die $self->message_method_undef_return( 'epoch_to_date' ) if ! defined $quote_f;
     return $quote_f;
 }
 
@@ -718,6 +864,7 @@ Example form the plugin C<App::DBBrowser::DB::mysql>:
 sub truncate {
     my ( $self, $quote_col, $precision ) = @_;
     my $quote_f = $self->{Plugin}->truncate( $quote_col, $precision );
+    die $self->message_method_undef_return( 'truncate' ) if ! defined $quote_f;
     return $quote_f;
 }
 
@@ -752,6 +899,7 @@ The sql bit length substatement.
 sub bit_length {
     my ( $self, $quote_col ) = @_;
     my $quote_f = $self->{Plugin}->bit_length( $quote_col );
+    die $self->message_method_undef_return( 'bit_length' ) if ! defined $quote_f;
     return $quote_f;
 }
 
@@ -784,6 +932,7 @@ Example form the plugin C<App::DBBrowser::DB::Pg>:
 sub char_length {
     my ( $self, $quote_col ) = @_;
     my $quote_f = $self->{Plugin}->char_length( $quote_col );
+    die $self->message_method_undef_return( 'char_length' ) if ! defined $quote_f;
     return $quote_f;
 }
 
