@@ -23,7 +23,7 @@ sub new {
     my ( $class, $opt ) = @_;
     $opt->{db_driver} = 'SQLite';
     $opt->{driver_prefix} = 'sqlite';
-    $opt->{plugin_api_version} = 1.2;
+    $opt->{plugin_api_version} = 1.5;
     bless $opt, $class;
 }
 
@@ -46,13 +46,19 @@ sub driver_prefix {
 }
 
 
-sub login_data {
-    my ( $self ) = @_;
-    return [];
-}
+#sub environment_variables {
+#    my ( $self ) = @_;
+#    return [];
+#}
 
 
-sub connect_attributes {
+#sub read_arguments {
+#    my ( $self ) = @_;
+#    return [];
+#}
+
+
+sub choose_arguments {
     my ( $self ) = @_;
     return [
         { name => 'sqlite_unicode',             default_index => 1, avail_values => [ 0, 1 ] },
@@ -69,7 +75,7 @@ sub get_db_handle {
         RaiseError => 1,
         AutoCommit => 1,
         ShowErrorStatement => 1,
-        %{$connect_parameter->{attributes}},
+        %{$connect_parameter->{chosen_arg}},
     } ) or die DBI->errstr;
     $dbh->sqlite_create_function( 'regexp', 3, sub {
             my ( $regex, $string, $case_sensitive ) = @_;
@@ -104,7 +110,7 @@ sub available_databases {
     my $dirs = $connect_parameter->{dir_sqlite};
     my $cache_key = $self->{db_plugin} . '_' . join ' ', @$dirs;
     my $auxil = App::DBBrowser::Auxil->new();
-    my $db_cache = $auxil->read_json( $self->{db_cache_file} );
+    my $db_cache = $auxil->__read_json( $self->{db_cache_file} );
     if ( $self->{sqlite_search} ) {
         delete $db_cache->{$cache_key};
     }
@@ -180,11 +186,34 @@ sub get_table_names {
 }
 
 
+sub primary_key_auto {
+    return "INTEGER PRIMARY KEY";
+}
+
+
+sub create_table {
+    my ( $self, $dbh, $table, $col_type ) = @_;
+    my $ct;
+    $ct .= "CREATE TABLE $table ( ";
+    $ct .= join ', ', map { $_->[0] . " " . $_->[1] } @$col_type;
+    $ct .= " )";
+    $dbh->do( $ct );
+    return;
+}
+
+
+sub drop_table {
+    my ( $self, $dbh, $table ) = @_;
+    $dbh->do( "DROP TABLE $table" );
+    return;
+}
+
+
 sub column_names_and_types {
-    my ( $self, $dbh, $db, $schema, $tables ) = @_;
+    my ( $self, $dbh, $db, $schema, $tables ) = @_; # qt_table
     my ( $col_names, $col_types );
     for my $table ( @$tables ) {
-        my $sth = $dbh->prepare( "SELECT * FROM " . $dbh->quote_identifier( undef, undef, $table ) );
+        my $sth = $dbh->prepare( "SELECT * FROM $table" );
         $col_names->{$table} = $sth->{NAME};
         $col_types->{$table} = $sth->{TYPE};
     }
