@@ -18,7 +18,7 @@ sub new {
     my ( $class, $opt ) = @_;
     $opt->{db_driver} = 'mysql';
     $opt->{driver_prefix} = 'mysql';
-    $opt->{plugin_api_version} = 1.1;
+    $opt->{plugin_api_version} = 1.5;
     bless $opt, $class;
 }
 
@@ -41,7 +41,13 @@ sub driver_prefix {
 }
 
 
-sub login_data {
+sub environment_variables {
+    my ( $self ) = @_;
+    return [ qw( DBI_DSN DBI_HOST DBI_PORT DBI_USER DBI_PASS ) ];
+}
+
+
+sub read_arguments {
     my ( $self ) = @_;
     return [
         { name => 'host', prompt => "Host",     keep_secret => 0 },
@@ -51,10 +57,12 @@ sub login_data {
     ];
 }
 
-sub connect_attributes {
+
+sub choose_arguments {
     my ( $self ) = @_;
     return [
-        { name => 'mysql_enable_utf8', default_index => 1, avail_values => [ 0, 1 ] },
+        { name => 'mysql_enable_utf8',        default_index => 1, avail_values => [ 0, 1 ] },
+        { name => 'mysql_bind_type_guessing', default_index => 1, avail_values => [ 0, 1 ] },
     ];
 }
 
@@ -65,19 +73,22 @@ sub get_db_handle {
         connect_parameter  => $connect_parameter,
         plugin_api_version => $self->plugin_api_version()
     } );
-    my $host   = $obj_db_cred->get_login( 'host' );
-    my $port   = $obj_db_cred->get_login( 'port' );
+    my $dsn;
+    if ( ! ( $connect_parameter->{use_env_var}{DBI_DSN} &&  exists $ENV{DBI_DSN} ) ) {
+        my $host = $obj_db_cred->get_login( 'host' );
+        my $port = $obj_db_cred->get_login( 'port' );
+        $dsn = "dbi:$self->{db_driver}:dbname=$db";
+        $dsn .= ";host=$host" if length $host;
+        $dsn .= ";port=$port" if length $port;
+    }
     my $user   = $obj_db_cred->get_login( 'user' );
     my $passwd = $obj_db_cred->get_login( 'pass' );
-    my $dsn  = "dbi:$self->{db_driver}:dbname=$db";
-    $dsn .= ";host=$host" if length $host;
-    $dsn .= ";port=$port" if length $port;
     my $dbh = DBI->connect( $dsn, $user, $passwd, {
         PrintError => 0,
         RaiseError => 1,
         AutoCommit => 1,
         ShowErrorStatement => 1,
-        %{$connect_parameter->{attributes}},
+        %{$connect_parameter->{chosen_arg}},
     } ) or die DBI->errstr;
     return $dbh;
 }
@@ -88,7 +99,7 @@ sub available_databases {
     return \@ARGV if @ARGV;
     my @regex_system_db = ( '^mysql$', '^information_schema$', '^performance_schema$' );
     my $stmt = "SELECT schema_name FROM information_schema.schemata";
-    if ( ! $self->{metadata} ) {
+    if ( ! $self->{add_metadata} ) {
         $stmt .= " WHERE " . join( " AND ", ( "schema_name NOT REGEXP ?" ) x @regex_system_db );
     }
     $stmt .= " ORDER BY schema_name";
@@ -96,9 +107,9 @@ sub available_databases {
     print $self->{clear_screen};
     print "DB: $info_database\n";
     my $dbh = $self->get_db_handle( $info_database, $connect_parameter );
-    my $databases = $dbh->selectcol_arrayref( $stmt, {}, $self->{metadata} ? () : @regex_system_db );
+    my $databases = $dbh->selectcol_arrayref( $stmt, {}, $self->{add_metadata} ? () : @regex_system_db );
     $dbh->disconnect(); #
-    if ( $self->{metadata} ) {
+    if ( $self->{add_metadata} ) {
         my $regexp = join '|', @regex_system_db;
         my $user_db   = [];
         my $system_db = [];
@@ -135,6 +146,29 @@ sub get_table_names {
 }
 
 
+sub primary_key_auto {
+    return "INT NOT NULL AUTO_INCREMENT PRIMARY KEY";
+}
+
+
+sub create_table {
+    my ( $self, $dbh, $table, $col_type, $add_primay_key ) = @_;
+    my $ct;
+    $ct .= "CREATE TABLE $table ( ";
+    $ct .= join ', ', map { $_->[0] . " " . $_->[1] } @$col_type;
+    $ct .= " )";
+    $dbh->do( $ct );
+    return;
+}
+
+
+sub drop_table {
+    my ( $self, $dbh, $table ) = @_;
+    $dbh->do( "DROP TABLE $table" );
+    return;
+}
+
+
 sub column_names_and_types {
     my ( $self, $dbh, $db, $schema, $tables ) = @_;
     my ( $col_names, $col_types );
