@@ -6,7 +6,7 @@ use strict;
 use 5.008003;
 no warnings 'utf8';
 
-our $VERSION = '0.999';
+our $VERSION = '1.016_01';
 
 use Clone                  qw( clone );
 use List::MoreUtils        qw( any );
@@ -27,12 +27,14 @@ sub new {
 
 
 sub __union_tables {
-    my ( $self, $dbh, $db, $schema, $data ) = @_;
+    my ( $self, $sql, $dbh, $data ) = @_;
+    my $db     = $sql->{print}{db};
+    my $schema = $sql->{print}{schema};
     my $no_lyt = Term::Choose->new();
     my $u = $data;
     if ( ! defined $u->{col_names} || ! defined $u->{col_types} ) {
         my $obj_db = App::DBBrowser::DB->new( $self->{info}, $self->{opt} );
-        ( $u->{col_names}, $u->{col_types} ) = $obj_db->column_names_and_types( $dbh, $db, $schema, $u->{tables} );
+        ( $u->{col_names}, $u->{col_types} ) = $obj_db->column_names_and_types( $dbh, $db, $schema, $u->{tables} ); # ###
     }
     my $union = {
         unused_tables => [ map { "- $_" } @{$u->{tables}} ],
@@ -55,12 +57,11 @@ sub __union_tables {
             $choices,
             { %{$self->{info}{lyt_stmt_v}}, prompt => $prompt, index => 1 }
         );
-        return if ! defined $idx_tbl;
-        my $union_table = $choices->[$idx_tbl];
-        if ( ! defined $union_table ) {
+        if ( ! defined $idx_tbl || ! defined $choices->[$idx_tbl] ) {
             return;
         }
-        elsif ( $union_table eq $info ) {
+        my $union_table = $choices->[$idx_tbl];
+        if ( $union_table eq $info ) {
             if ( ! defined $u->{tables_info} ) {
                 $u->{tables_info} = $self->__get_tables_info( $dbh, $db, $schema, $u, 'union' );
             }
@@ -166,28 +167,31 @@ sub __union_tables {
             last UNION_TABLE;
         }
     }
+
+
     # column names in the result-set of a UNION are taken from the first query.
     my $first_table = $union->{used_tables}[0];
-    $union->{pr_columns} = $union->{used_cols}{$first_table};
-    for my $col ( @{$union->{pr_columns}} ) {
-        $union->{qt_columns}{$col} = $dbh->quote_identifier( $col );
+    $sql->{print}{columns} = $union->{used_cols}{$first_table};
+    for my $col ( @{$sql->{print}{columns}} ) {
+        $sql->{quote}{columns}{$col} = $dbh->quote_identifier( $col );
     }
-    $union->{quote}{stmt} = "SELECT * FROM (";
+    $sql->{quote}{col_stmt} = "*";
     my $c;
+    $sql->{quote}{table} = "(";
     for my $table ( @{$union->{used_tables}} ) {
         $c++;
-        $union->{quote}{stmt} .= " SELECT ";
-        $union->{quote}{stmt} .= join( ', ', map { $dbh->quote_identifier( $_ ) } @{$union->{used_cols}{$table}} );
-        $union->{quote}{stmt} .= " FROM " . $dbh->quote_identifier( undef, $schema, $table );
-        $union->{quote}{stmt} .= $c < @{$union->{used_tables}} ? " UNION ALL " : " )";
+        $sql->{quote}{table} .= " SELECT ";
+        $sql->{quote}{table} .= join( ', ', map { $dbh->quote_identifier( $_ ) } @{$union->{used_cols}{$table}} );
+        $sql->{quote}{table} .= " FROM " . $dbh->quote_identifier( undef, $schema, $table );
+        $sql->{quote}{table} .= $c < @{$union->{used_tables}} ? " UNION ALL " : " )";
     }
-    if ( $self->{union_all} ) {
-        $union->{quote}{stmt} .= " AS " . $dbh->quote_identifier( 'UNION_ALL_TABLES' );
+    if ( $self->{union_all} ) {  # alias: required if mysql, Pg, ...
+        $sql->{quote}{table} .= " AS UNION_ALL_TABLES";
     }
     else {
-        $union->{quote}{stmt} .= " AS " . $dbh->quote_identifier( 'UNION_SELECTED_TABLES' );
+        $sql->{quote}{table} .= " AS UNION_SELECTED_TABLES";
     }
-    return $union;
+    return 1;
 }
 
 
@@ -238,14 +242,17 @@ sub __get_tables_info {
     my $tables_info = {};
     my $sth;
     my $obj_db = App::DBBrowser::DB->new( $self->{info}, $self->{opt} );
-    my ( $pk, $fk ) = $obj_db->primary_and_foreign_keys( $dbh, $db, $schema, $u_or_j->{tables} );
+    my ( $pk, $fk ) = $obj_db->primary_and_foreign_keys( $dbh, $db, $schema, $u_or_j->{tables} ); # ###
     for my $table ( @{$u_or_j->{tables}} ) {
         push @{$tables_info->{$table}}, 'Table => ' . $table;
-        if ( defined $u_or_j->{col_names} ) {
+        if ( defined $u_or_j->{col_types}{$table} ) {
             push @{$tables_info->{$table}}, join( ' | ', map {
                             lc( $u_or_j->{col_types}{$table}[$_] )
                         . ' ' . $u_or_j->{col_names}{$table}[$_]   } 0 .. $#{$u_or_j->{col_names}{$table}} );
         }
+        else {
+            push @{$tables_info->{$table}}, join ' | ',  @{$u_or_j->{col_names}{$table}};
+        }
         if ( $type eq 'join' ) {
             if ( defined $pk && @{$pk->{$table}} ) {
                 push @{$tables_info->{$table}}, 'PK: primary key (' . join( ',', @{$pk->{$table}} ) . ')';
@@ -262,9 +269,6 @@ sub __get_tables_info {
                 }
             }
         }
-        if ( @{$tables_info->{$table}} == 1 ) {
-            push @{$tables_info->{$table}}, 'No INFO available.';
-        }
     }
     return $tables_info;
 }
@@ -287,18 +291,23 @@ sub __print_tables_info {
 
 
 sub __join_tables {
-    my ( $self, $dbh, $db, $schema, $data ) = @_;
+    my ( $self, $sql, $dbh, $data ) = @_;
+    my $db     = $sql->{print}{db};
+    my $schema = $sql->{print}{schema};
     my $stmt_v = Term::Choose->new( $self->{info}{lyt_stmt_v} );
-    my $join = {};
-    $join->{quote}{stmt} = "SELECT * FROM";
-    $join->{print}{stmt} = "SELECT * FROM";
     my $j = $data;
     if ( ! defined $j->{col_names} || ! defined $j->{col_types} ) {
         my $obj_db = App::DBBrowser::DB->new( $self->{info}, $self->{opt} );
-        ( $j->{col_names}, $j->{col_types} ) = $obj_db->column_names_and_types( $dbh, $db, $schema, $j->{tables} );
+        ( $j->{col_names}, $j->{col_types} ) = $obj_db->column_names_and_types( $dbh, $db, $schema, $j->{tables} ); # ###
     }
+    my $join = {};
 
     MASTER: while ( 1 ) {
+        $join = {};
+        $join->{quote}{stmt} = "SELECT * FROM";
+        $join->{print}{stmt} = "SELECT * FROM";
+        $join->{primary_keys} = [];
+        $join->{foreign_keys} = [];
         my @tables = map { "- $_" } @{$j->{tables}};
         my $info = '  INFO';
         my @pre = ( undef );
@@ -309,10 +318,10 @@ sub __join_tables {
             $choices,
             { prompt => 'Choose MASTER table:', index => 1 }
         );
-        return if ! defined $idx;
-        my $master = $choices->[$idx];
-        return if ! defined $master;
-        if ( $master eq $info ) {
+        if ( ! defined $idx || ! defined $choices->[$idx] ) {
+            return;
+        }
+        if ( $choices->[$idx] eq $info ) {
             if ( ! defined $j->{tables_info} ) {
                 $j->{tables_info} = $self->__get_tables_info( $dbh, $db, $schema, $j, 'join' );
             }
@@ -325,23 +334,22 @@ sub __join_tables {
             next MASTER;
         }
         $idx -= @pre;
-        splice( @tables, $idx, 1 );
-        $master =~ s/^-\s//;
+        ( my $master = splice( @tables, $idx, 1 ) ) =~ s/^-\s//;
         $join->{used_tables}  = [ $master ];
         $join->{avail_tables} = [ @tables ];
-        $join->{quote}{stmt}  = "SELECT * FROM " . $dbh->quote_identifier( undef, $schema, $master );
-        $join->{print}{stmt}  = "SELECT * FROM " .                                         $master  ;
-        $join->{primary_keys} = [];
-        $join->{foreign_keys} = [];
+        $join->{table_alias} = 'a';
+        $join->{quote}{stmt}  = "SELECT * FROM " . $dbh->quote_identifier( undef, $schema, $master ) . " AS " . $join->{table_alias};
+        $join->{print}{stmt}  = "SELECT * FROM " .                 $master . " AS " . $join->{table_alias};
+        $join->{alias}{$master} = $join->{table_alias};
         my $backup_master = clone( $join );
 
         JOIN: while ( 1 ) {
-            my ( $idx, $slave );
+            my $idx;
+            my $enough_slaves = '  Enough TABLES';
+            my @pre = ( undef, $enough_slaves );
             my $backup_join = clone( $join );
 
             SLAVE: while ( 1 ) {
-                my $enough_slaves = '  Enough TABLES';
-                my @pre = ( undef, $enough_slaves );
                 my $choices = [ @pre, @{$join->{avail_tables}}, $info ];
                 $self->__print_join_statement( $join->{print}{stmt} );
                 # Choose
@@ -349,15 +357,8 @@ sub __join_tables {
                     $choices,
                     { prompt => 'Add a SLAVE table:', index => 1, undef => $self->{info}{_reset} }
                 );
-                if ( defined $idx ) {
-                    $slave = $choices->[$idx];
-                    $idx -= @pre;
-                }
-                if ( ! defined $slave ) {
+                if ( ! defined $idx || ! defined $choices->[$idx] ) {
                     if ( @{$join->{used_tables}} == 1 ) {
-                        @tables = map { "- $_" } @{$j->{tables}};
-                        $join->{quote}{stmt} = "SELECT * FROM";
-                        $join->{print}{stmt} = "SELECT * FROM";
                         next MASTER;
                     }
                     else {
@@ -365,10 +366,10 @@ sub __join_tables {
                         next JOIN;
                     }
                 }
-                elsif ( $slave eq $enough_slaves ) {
+                elsif ( $choices->[$idx] eq $enough_slaves ) {
                     last JOIN;
                 }
-                elsif ( $slave eq $info ) {
+                elsif ( $choices->[$idx] eq $info ) {
                     if ( ! defined $j->{tables_info} ) {
                         $j->{tables_info} = $self->__get_tables_info( $dbh, $db, $schema, $j, 'join' );
                     }
@@ -384,19 +385,21 @@ sub __join_tables {
                     last SLAVE;
                 }
             }
-            splice( @{$join->{avail_tables}}, $idx, 1 );
-            $slave =~ s/^-\s//;
-            $join->{quote}{stmt} .= " LEFT OUTER JOIN " . $dbh->quote_identifier( undef, $schema, $slave ) . " ON";
-            $join->{print}{stmt} .= " LEFT OUTER JOIN " .                                         $slave   . " ON";
+            $idx -= @pre;
+            ( my $slave = splice( @{$join->{avail_tables}}, $idx, 1 ) ) =~ s/^-\s//;
+            $join->{table_alias}++;
+            $join->{quote}{stmt} .= " LEFT OUTER JOIN " . $dbh->quote_identifier( undef, $schema, $slave ) . " AS " . $join->{table_alias} . " ON";
+            $join->{print}{stmt} .= " LEFT OUTER JOIN " .                                         $slave .   " AS " . $join->{table_alias} . " ON";
+            $join->{alias}{$slave} = $join->{table_alias};
             my %avail_pk_cols;
             for my $used_table ( @{$join->{used_tables}} ) {
                 for my $col ( @{$j->{col_names}{$used_table}} ) {
-                    $avail_pk_cols{ $used_table . '.' . $col } = $dbh->quote_identifier( undef, $used_table, $col );
+                    $avail_pk_cols{ $join->{alias}{$used_table} . '.' . $col } = $dbh->quote_identifier( undef, $join->{alias}{$used_table}, $col );
                 }
             }
             my %avail_fk_cols;
             for my $col ( @{$j->{col_names}{$slave}} ) {
-                $avail_fk_cols{ $slave . '.' . $col } = $dbh->quote_identifier( undef, $slave, $col );
+                $avail_fk_cols{ $join->{alias}{$slave} . '.' . $col } = $dbh->quote_identifier( undef, $join->{alias}{$slave}, $col );
             }
             my $AND = '';
 
@@ -447,34 +450,27 @@ sub __join_tables {
         last MASTER;
     }
 
-    #my @not_unique_col;
-    #my %seen;
-    #for my $table (@{$join->{used_tables}} ) {
-    #    for my $col ( @{$j->{col_names}{$table}} ) {
-    #        $seen{$col}++;
-    #        push @not_unique_col, $col if $seen{$col} == 2;
-    #    }
-    #}
     my $col_stmt = '';
     for my $table ( @{$join->{used_tables}} ) {
         for my $col ( @{$j->{col_names}{$table}} ) {
-            my $col_qt = $dbh->quote_identifier( undef, $table, $col );
-            my $col_pr = $col;
+            my $col_qt = $dbh->quote_identifier( undef, $join->{alias}{$table}, $col );
             if ( any { $_ eq $col_qt } @{$join->{foreign_keys}} ) {
                 next;
             }
+            my $col_pr = $join->{alias}{$table} . '.' . $col;
             #if ( any { $_ eq $col_pr } @not_unique_col ) {
-                $col_pr .= '_' . $table;
-                $col_qt .= " AS " . $dbh->quote_identifier( $col_pr );
+                $col_qt .= " AS " . $dbh->quote_identifier( $col . '-' . $table );
             #}
-            push @{$join->{pr_columns}}, $col_pr;
-            $join->{qt_columns}{$col_pr} = $col_qt;
+            push @{$sql->{print}{columns}}, $col_pr;
+            $sql->{quote}{columns}{$col_pr} = $col_qt;
             $col_stmt .= ', ' . $col_qt;
         }
     }
     $col_stmt =~ s/^,\s//;
-    $join->{quote}{stmt} =~ s/\s\*\s/ $col_stmt /;
-    return $join;
+    my ( $qt_table ) = $join->{quote}{stmt} =~ /^SELECT\s\*\sFROM\s(.*)\z/;
+    $sql->{quote}{col_stmt} = $col_stmt;
+    $sql->{quote}{table} = $qt_table;
+    return 1;
 }
 
 
