@@ -6,7 +6,7 @@ use strict;
 use 5.008003;
 no warnings 'utf8';
 
-our $VERSION = '0.999';
+our $VERSION = '1.016_01';
 
 use File::Basename        qw( basename fileparse );
 use File::Spec::Functions qw( catfile );
@@ -53,11 +53,12 @@ sub defaults {
             tab_width            => 2,
             undef                => '',
             binary_string        => 'BNRY',
+            binary_filter        => 0,
         },
         insert => {
-            input_modes          => [ 'Cols', 'Multirow', 'File' ],
-            csv_read             => 0,
-            encoding_csv_file    => 'UTF-8',
+            input_modes          => [ 'Cols', 'Rows', 'Multi-row', 'File' ],
+            parse_mode           => 0,
+            file_encoding        => 'UTF-8',
             max_files            => 15,
         # Text::CSV:
             sep_char             => ',',
@@ -70,9 +71,9 @@ sub defaults {
             blank_is_undef       => 1,
             binary               => 1,
             empty_is_undef       => 0,
-        # Text::ParseWords:
-            delim                => ',',
-            keep                 => 0,
+        # split:
+            i_r_s                => '\n',
+            i_f_s                => ',',
         }
     };
     return $defaults                   if ! $section;
@@ -85,28 +86,24 @@ sub __sub_menus_insert {
     my ( $self, $group ) = @_;
     my $sub_menus_insert = {
         main_insert => [
-            { name => 'input_modes',             text => "- Input modes",      section => 'insert' },
-            { name => 'csv_read',                text => "- Parse module",     section => 'insert' },
-            { name => 'encoding_csv_file',       text => "- File encoding",    section => 'insert' },
-            { name => '_module_text_csv',        text => "- Text::CSV",        section => 'insert' },
-            { name => '_module_text_parsewords', text => "- Text::ParseWords", section => 'insert' },
-            { name => 'max_files',               text => "- File history",     section => 'insert' },
+            { name => 'input_modes',       text => "- Read",          section => 'insert' },
+            { name => 'parse_mode',        text => "- Parse-mode",    section => 'insert' },
+            { name => '_module_Text_CSV',  text => "- conf T::CSV",   section => 'insert' },
+            { name => '_parse_with_split', text => "- conf 'split'",  section => 'insert' },
+            { name => 'file_encoding',     text => "- File Encoding", section => 'insert' },
+            { name => 'max_files',         text => "- File History",  section => 'insert' },
         ],
-        _module_text_csv => [
+        _module_Text_CSV => [
             { name => '_csv_char',    text => "- *_char attributes", section => 'insert' },
             { name => '_options_csv', text => "-  Other attributes", section => 'insert' },
         ],
-        _module_text_parsewords => [
-            { name => 'delim', text => "- \$delim", section => 'insert' },
-            { name => 'keep',  text => "- \$keep",  section => 'insert' },
-        ],
     };
     return $sub_menus_insert->{$group};
 }
 
 
 sub __config_insert {
-    my ( $self, $write_to_file ) = @_;
+    my ( $self ) = @_;
     my $old_idx = 0;
     my $backup_old_idx = 0;
     my $group  = 'main_insert';
@@ -115,14 +112,19 @@ sub __config_insert {
         my $sub_menu_insert = $self->__sub_menus_insert( $group );
 
         OPTION_INSERT: while ( 1 ) {
-            my @pre    = ( undef );
-            my @real   = map( $_->{text}, @$sub_menu_insert );
+            my $prompt;
+            if ( $group =~ /^_module_(.+)\z/ ) {
+                ( my $name = $1 ) =~ s/_/::/g;
+                $prompt = '"' . $name . '"';
+            }
+            my @pre     = ( undef );
+            my $choices = [ @pre, map( $_->{text}, @$sub_menu_insert ) ];
             # Choose
             my $idx = choose(
-                [ @pre, @real ],
-                { %{$self->{info}{lyt_3}}, index => 1, default => $old_idx, undef => $self->{info}{conf_back} }
+                $choices,
+                { %{$self->{info}{lyt_3}}, index => 1, default => $old_idx, undef => $self->{info}{back_short}, prompt => $prompt }
             );
-            if ( ! $idx ) {
+            if ( ! defined $idx || ! defined $choices->[$idx] ) {
                 if ( $group =~ /^_module_/ ) {
                     $old_idx = $backup_old_idx;
                     $group = 'main_insert';
@@ -131,7 +133,7 @@ sub __config_insert {
                 else {
                     if ( $self->{info}{write_config} ) {
                         $self->__write_config_files();
-                        delete $self->{info}{write_config} if $write_to_file;
+                        delete $self->{info}{write_config};
                     }
                     return
                 }
@@ -156,21 +158,26 @@ sub __config_insert {
                 $group = $option;
                 redo GROUP_INSERT;
             }
-            my $section = $sub_menu_insert->[$idx - @pre]{section};
-            my $no_yes  = [ 'NO', 'YES' ];
+            my $section  = $sub_menu_insert->[$idx - @pre]{section};
+            my $opt_type = 'opt';
+            my $no_yes   = [ 'NO', 'YES' ];
             if ( $option eq 'input_modes' ) {
-                    my $available = [ 'Cols', 'Rows', 'Multirow', 'File' ];
-                    $self->__opt_choose_a_list( $section, $option, $available );
+                    my $available = [ 'Cols', 'Rows', 'Multi-row', 'File' ];
+                    my $prompt = 'Input Modes:';
+                    $self->__opt_choose_a_list( $opt_type, $section, $option, $available, $prompt );
             }
-            elsif ( $option eq 'csv_read' ) {
-                my $prompt = 'Parsing CSV files: ';
-                my $list = [ 'Text::CSV', 'Text::ParseWords', 'Spreadsheet::Read' ];
+            elsif ( $option eq 'parse_mode' ) {
+                my $prompt = 'Parsing CSV files';
+                my $list = [ 'Text::CSV', 'split', 'Spreadsheet::Read' ];
                 my $sub_menu = [ [ $option, "  Use", $list ] ];
-                $self->__opt_choose_multi( $section, $sub_menu, $prompt );
+                $self->__opt_choose_multi( $opt_type, $section, $sub_menu, $prompt );
             }
-            elsif ( $option eq 'encoding_csv_file' ) {
+            elsif ( $option eq 'file_encoding' ) {
+                my $items = [
+                    { name => 'file_encoding', prompt => "file_encoding" },
+                ];
                 my $prompt = 'Encoding CSV files';
-                $self->__opt_readline( 'insert', $option, $prompt );
+                $self->__group_readline( $opt_type, $section, $items, $prompt );
             }
             elsif ( $option eq '_csv_char' ) {
                 my $items = [
@@ -178,11 +185,11 @@ sub __config_insert {
                     { name => 'quote_char',  prompt => "quote_char " },
                     { name => 'escape_char', prompt => "escape_char" },
                 ];
-                my $prompt = 'Text::CSV:';
-                $self->__group_readline( $section, $items, $prompt );
+                my $prompt = '"Text::CSV"';
+                $self->__group_readline( $opt_type, $section, $items, $prompt );
             }
             elsif ( $option eq '_options_csv' ) {
-                my $prompt = 'Text::CSV:';
+                my $prompt = '"Text::CSV"';
                 my $sub_menu = [
                     [ 'allow_loose_escapes', "- allow_loose_escapes", [ 'NO', 'YES' ] ],
                     [ 'allow_loose_quotes',  "- allow_loose_quotes",  [ 'NO', 'YES' ] ],
@@ -190,22 +197,20 @@ sub __config_insert {
                     [ 'blank_is_undef',      "- blank_is_undef",      [ 'NO', 'YES' ] ],
                     [ 'empty_is_undef',      "- empty_is_undef",      [ 'NO', 'YES' ] ],
                 ];
-                $self->__opt_choose_multi( $section, $sub_menu, $prompt );
+                $self->__opt_choose_multi( $opt_type, $section, $sub_menu, $prompt );
             }
-            elsif ( $option eq 'delim' ) {
-                my $prompt = 'Text::ParseWords delimiter (regexp)';
-                $self->__opt_readline( $section, $option, $prompt );
-            }
-            elsif ( $option eq 'keep' ) {
-                my $prompt = 'Text::ParseWords: ';
-                my $list = $no_yes;
-                my $sub_menu = [ [ $option, "  Enable option '\$keep'", $list ] ];
-                $self->__opt_choose_multi( $section, $sub_menu, $prompt );
+            elsif ( $option eq '_parse_with_split' ) {
+                my $items = [
+                    { name => 'i_r_s', prompt => "IRS" },
+                    { name => 'i_f_s', prompt => "IFS" },
+                ];
+                my $prompt = 'Separators (regexp)';
+                $self->__group_readline( $opt_type, $section, $items, $prompt );
             }
             elsif ( $option eq 'max_files' ) {
                 my $digits = 3;
-                my $prompt = 'Save the last x input file names';
-                $self->__opt_number_range( $section, $option, $prompt, $digits );
+                my $prompt = '"Max file history"';
+                $self->__opt_number_range( $opt_type, $section, $option, $prompt, $digits );
             }
             else { die "Unknown option: $option" }
         }
@@ -235,7 +240,6 @@ sub __menus {
             { name => 'mouse',         text => "- Mouse Mode",  section => 'table' },
         ],
         config_sql => [
-            { name => 'max_rows',     text => "- Max Rows",    section => 'table' },
             { name => 'metadata',     text => "- Metadata",    section => 'G' },
             { name => 'operators',    text => "- Operators",   section => 'G' },
             { name => 'lock_stmt',    text => "- Lock Mode",   section => 'G' },
@@ -243,6 +247,7 @@ sub __menus {
 
         ],
         config_output => [
+            { name => 'max_rows',      text => "- Max Rows",    section => 'table' },
             { name => 'min_col_width', text => "- Colwidth",    section => 'table' },
             { name => 'progress_bar',  text => "- ProgressBar", section => 'table' },
             { name => 'tab_width',     text => "- Tabwidth",    section => 'table' },
@@ -253,7 +258,7 @@ sub __menus {
 }
 
 
-sub set_options {
+sub __set_options {
     my ( $self ) = @_;
     my $group = 'main';
     my $backup_old_idx = 0;
@@ -263,17 +268,15 @@ sub set_options {
         my $menu = $self->__menus( $group );
 
         OPTION: while ( 1 ) {
-            my $back =          $group eq 'main' ? $self->{info}{_quit}     : $self->{info}{conf_back};
+            my $back =          $group eq 'main' ? $self->{info}{_quit}     : $self->{info}{back_short};
             my @pre  = ( undef, $group eq 'main' ? $self->{info}{_continue} : () );
-            my @real = map( $_->{text}, @$menu );
+            my $choices = [ @pre, map( $_->{text}, @$menu ) ];
             # Choose
             my $idx = choose(
-                [ @pre, @real ],
+                $choices,
                 { %{$self->{info}{lyt_3}}, index => 1, default => $old_idx, undef => $back }
             );
-            exit if ! defined $idx;
-            my $option = $idx <= $#pre ? $pre[$idx] : $menu->[$idx - @pre]{name};
-            if ( ! defined $option ) {
+            if ( ! defined $idx || ! defined $choices->[$idx] ) {
                 if ( $group =~ /^config_/ ) {
                     $old_idx = $backup_old_idx;
                     $group = 'main';
@@ -300,34 +303,35 @@ sub set_options {
                     next OPTION;
                 }
             }
+            my $option = $idx <= $#pre ? $pre[$idx] : $menu->[$idx - @pre]{name};
             if ( $option eq 'config_insert' ) {
-                $self->__config_insert( 1 );
+                $backup_old_idx = $old_idx;
+                $self->__config_insert();
                 $old_idx = $backup_old_idx;
                 $group = 'main';
                 redo GROUP;
             }
-            if ( $option =~ /^config_/ ) {
+            elsif ( $option =~ /^config_/ ) {
                 $backup_old_idx = $old_idx;
                 $old_idx = 0;
                 $group = $option;
                 redo GROUP;
             }
-
-            if ( $option eq $self->{info}{_continue} ) {
+            elsif ( $option eq $self->{info}{_continue} ) {
                 if ( $self->{info}{write_config} ) {
                     $self->__write_config_files();
                     delete $self->{info}{write_config};
                 }
                 return $self->{opt}, $self->{db_opt};
             }
-            if ( $option eq 'help' ) {
+            elsif ( $option eq 'help' ) {
                 require Pod::Usage;
                 Pod::Usage::pod2usage( {
                     -exitval => 'NOEXIT',
                     -verbose => 2 } );
                 next OPTION;
             }
-            if ( $option eq 'path' ) {
+            elsif ( $option eq 'path' ) {
                 my $version = 'version';
                 my $bin     = '  bin  ';
                 my $app_dir = 'app-dir';
@@ -340,88 +344,97 @@ sub set_options {
                 print_hash( $path, { keys => $names, preface => ' Close with ENTER' } );
                 next OPTION;
             }
-            if ( $option eq '_db_defaults' ) {
-                $self->database_setting();
+            elsif ( $option eq '_db_defaults' ) {
+                $self->__database_setting();
                 next OPTION;
             }
-            my $section     = $menu->[$idx - @pre]{section};
-            my $ref_section = $self->{opt}{$section};
-            my $no_yes      = [ 'NO', 'YES' ];
-            if ( $option eq 'tab_width' ) {
-                my $digits = 3;
-                my $prompt = 'Tab width';
-                $self->__opt_number_range( $ref_section, $option, $prompt, $digits );
-            }
-            elsif ( $option eq 'min_col_width' ) {
-                my $digits = 3;
-                my $prompt = 'Minimum Column width';
-                $self->__opt_number_range( $ref_section, $option, $prompt, $digits );
-            }
-            elsif ( $option eq 'undef' ) {
-                my $prompt = 'Print replacement for undefined table vales';
-                $self->__opt_readline( $ref_section, $option, $prompt );
-            }
-            elsif ( $option eq 'progress_bar' ) {
-                my $digits = 7;
-                my $prompt = '"Threshold ProgressBar"';
-                $self->__opt_number_range( $ref_section, $option, $prompt, $digits );
-            }
-            elsif ( $option eq 'max_rows' ) {
-                my $digits = 7;
-                my $prompt = '"Max rows"';
-                $self->__opt_number_range( $ref_section, $option, $prompt, $digits );
-            }
-            elsif ( $option eq 'lock_stmt' ) {
-                my $prompt = 'SQL statement: ';
-                my $list = [ 'Lk0', 'Lk1' ];
-                my $sub_menu = [ [ $option, "  Lock Mode", $list ] ];
-                $self->__opt_choose_multi( $ref_section, $sub_menu, $prompt );
-            }
-            elsif ( $option eq 'metadata' ) {
-                my $prompt = 'DB/schemas/tables: ';
-                my $list = $no_yes;
-                my $sub_menu = [ [ $option, "  Add metadata", $list ] ];
-                $self->__opt_choose_multi( $ref_section, $sub_menu, $prompt );
-            }
-            elsif ( $option eq '_parentheses' ) {
-                my $sub_menu = [
-                    [ 'parentheses_w', "- Parens in WHERE",     [ 'NO', '(YES', 'YES(' ] ],
-                    [ 'parentheses_h', "- Parens in HAVING TO", [ 'NO', '(YES', 'YES(' ] ],
-                ];
-                $self->__opt_choose_multi( $ref_section, $sub_menu );
-            }
-            elsif ( $option eq 'operators' ) {
-                my $available = $self->{info}{avail_operators};
-                $self->__opt_choose_a_list( $ref_section, $option, $available );
-            }
-            elsif ( $option eq 'db_plugins' ) {
+            my $opt_type = 'opt';
+            my $section  = $menu->[$idx - @pre]{section};
+            my $no_yes   = [ 'NO', 'YES' ];
+            if ( $option eq 'db_plugins' ) {
                 my %installed_db_driver;
                 for my $dir ( @INC ) {
                     my $glob_pattern = catfile $dir, 'App', 'DBBrowser', 'DB', '*.pm';
                     map { $installed_db_driver{( fileparse $_, '.pm' )[0]}++ } glob $glob_pattern;
                 }
-                $self->__opt_choose_a_list( $ref_section, $option, [ sort keys %installed_db_driver ] );
+                my $prompt = 'Choose DB plugins:';
+                $self->__opt_choose_a_list( $opt_type, $section, $option, [ sort keys %installed_db_driver ], $prompt );
+                $self->__read_db_config_files();
+            }
+            elsif ( $option eq 'tab_width' ) {
+                my $digits = 3;
+                my $prompt = '"Tab width"';
+                $self->__opt_number_range( $opt_type, $section, $option, $prompt, $digits );
+            }
+            elsif ( $option eq 'min_col_width' ) {
+                my $digits = 3;
+                my $prompt = '"Min column width"';
+                $self->__opt_number_range( $opt_type, $section, $option, $prompt, $digits );
+            }
+            elsif ( $option eq 'undef' ) {
+                my $items = [
+                    { name => 'undef', prompt => "undef" },
+                ];
+                my $prompt = 'Print replacement for undefined table values.';
+                $self->__group_readline( $opt_type, $section, $items, $prompt );
+
+            }
+            elsif ( $option eq 'progress_bar' ) {
+                my $digits = 7;
+                my $prompt = '"Threshold ProgressBar"';
+                $self->__opt_number_range( $opt_type, $section, $option, $prompt, $digits );
+            }
+            elsif ( $option eq 'max_rows' ) {
+                my $digits = 7;
+                my $prompt = '"Max rows"';
+                $self->__opt_number_range( $opt_type, $section, $option, $prompt, $digits );
+            }
+            elsif ( $option eq 'lock_stmt' ) {
+                my $prompt = 'SQL statement: ';
+                my $list = [ 'Lk0', 'Lk1' ];
+                my $sub_menu = [ [ $option, "  Lock Mode", $list ] ];
+                $self->__opt_choose_multi( $opt_type, $section, $sub_menu, $prompt );
+            }
+            elsif ( $option eq 'metadata' ) {
+                my $prompt = 'DB/schemas/tables: ';
+                my $list = $no_yes;
+                my $sub_menu = [ [ $option, "  Add metadata", $list ] ];
+                $self->__opt_choose_multi( $opt_type, $section, $sub_menu, $prompt );
+            }
+            elsif ( $option eq '_parentheses' ) {
+                my $sub_menu = [
+                    [ 'parentheses_w', "- Parens in WHERE",     [ 'NO', 'YES' ] ],
+                    [ 'parentheses_h', "- Parens in HAVING TO", [ 'NO', 'YES' ] ],
+                ];
+                $self->__opt_choose_multi( $opt_type, $section, $sub_menu );
+            }
+            elsif ( $option eq 'operators' ) {
+                my $available = $self->{info}{avail_operators};
+                my $prompt = 'Choose operators:';
+                $self->__opt_choose_a_list( $opt_type, $section, $option, $available, $prompt );
             }
             elsif ( $option eq 'mouse' ) {
-                my $prompt = 'Mouse mode: ';
+                my $prompt = 'Choose: ';
                 my $list = [ 0, 1, 2, 3, 4 ];
                 my $sub_menu = [ [ $option, "  Mouse mode", $list ] ];
-                $self->__opt_choose_multi( $ref_section, $sub_menu, $prompt );
+                $self->__opt_choose_multi( $opt_type, $section, $sub_menu, $prompt );
             }
             elsif ( $option eq '_menu_memory' ) {
+                my $prompt = 'Choose: ';
                 my $sub_menu = [
                     [ 'menus_config_memory', "- Config Menus", [ 'Simple', 'Memory' ] ],
                     [ 'menu_sql_memory',     "- SQL    Menu",  [ 'Simple', 'Memory' ] ],
                     [ 'menus_db_memory',     "- DB     Menus", [ 'Simple', 'Memory' ] ],
                 ];
-                $self->__opt_choose_multi( $ref_section, $sub_menu );
+                $self->__opt_choose_multi( $opt_type, $section, $sub_menu, $prompt );
             }
             elsif ( $option eq '_table_expand' ) {
+                my $prompt = 'Choose: ';
                 my $sub_menu = [
-                    [ 'table_expand', "- Table Rows",   [ 'Simple', 'Expand'    ] ],
                     [ 'keep_header',  "- Table Header", [ 'Simple', 'Each page' ] ],
+                    [ 'table_expand', "- Table Rows",   [ 'Simple', 'Expand fast back', 'Expand' ] ],
                 ];
-                $self->__opt_choose_multi( $ref_section, $sub_menu );
+                $self->__opt_choose_multi( $opt_type, $section, $sub_menu, $prompt );
             }
             else { die "Unknown option: $option" }
         }
@@ -430,106 +443,75 @@ sub set_options {
 
 
 sub __opt_choose_multi {
-    my ( $self, $ref_section, $sub_menu, $prompt ) = @_;
-    my $val = $ref_section;
-    my $changed = choose_multi( $sub_menu, $val, {  prompt => $prompt } );
+    my ( $self, $opt_type, $section, $sub_menu, $prompt ) = @_;
+    my $changed = choose_multi( $sub_menu, $self->{$opt_type}{$section}, { prompt => $prompt } );
     return if ! $changed;
     $self->{info}{write_config}++;
 }
 
 
 sub __opt_choose_a_list {
-    my ( $self, $ref_section, $option, $available, $index ) = @_;
-    my $current = $ref_section->{$option};
+    my ( $self, $opt_type, $section, $option, $available, $prompt ) = @_;
+    my $current = $self->{$opt_type}{$section}{$option};
     # Choose_list
-    my $list = choose_a_subset( $available, { current => $current, index => $index } );
+    my $list = choose_a_subset( $available, { prompt => $prompt, current => $current, index => 0 } );
     return if ! defined $list;
     return if ! @$list;
-    $ref_section->{$option} = $list;
+    $self->{$opt_type}{$section}{$option} = $list;
     $self->{info}{write_config}++;
     return;
 }
 
 
 sub __opt_number_range {
-    my ( $self, $ref_section, $option, $prompt, $digits ) = @_;
-    my $current = $ref_section->{$option};
+    my ( $self, $opt_type, $section, $option, $prompt, $digits ) = @_;
+    my $current = $self->{$opt_type}{$section}{$option};
     $current = insert_sep( $current, $self->{opt}{G}{thsd_sep} );
     # Choose_a_number
     my $choice = choose_a_number( $digits, { name => $prompt, current => $current } );
     return if ! defined $choice;
-    $ref_section->{$option} = $choice eq '--' ? undef : $choice;
+    $self->{$opt_type}{$section}{$option} = $choice eq '--' ? undef : $choice;
     $self->{info}{write_config}++;
     return;
 }
 
 
 sub __group_readline {
-    my ( $self, $ref_section, $items, $prompt ) = @_;
-    my $old_idx = 0;
-
-    OPTION: while ( 1 ) {
-        my @pre = ( undef );
-        my @real = map {
-                '- '
-            . $_->{prompt}
-            . ( $ref_section->{$_->{name}} ? ": $ref_section->{$_->{name}}" : ":" )
-        } @{$items};
-        my $choices = [ @pre, @real ];
-        # Choose
-        my $idx = choose(
-            $choices,
-            { %{$self->{info}{lyt_3}}, index => 1, default => $old_idx,
-              prompt => $prompt, undef => $self->{info}{conf_back} }
-        );
-        if ( ! $idx ) {
-            return;
+    my ( $self, $opt_type, $section, $items, $prompt ) = @_;
+    my $list = [ map {
+        [
+            exists $_->{prompt} ? $_->{prompt} : $_->{name},
+            $self->{$opt_type}{$section}{$_->{name}}
+        ]
+    } @{$items} ];
+    my $trs = Term::ReadLine::Simple->new();
+    my $new_list = $trs->fill_form(
+        $list,
+        { prompt => $prompt, auto_up => 2, confirm => $self->{info}{_confirm}, back => $self->{info}{_back} }
+    );
+    if ( $new_list ) {
+        for my $i ( 0 .. $#$items ) {
+            $self->{$opt_type}{$section}{$items->[$i]{name}} = $new_list->[$i][1];
         }
-        if ( $self->{opt}{G}{menus_config_memory} ) {
-            if ( $old_idx == $idx ) {
-                $old_idx = 0;
-                next OPTION;;
-            }
-            else {
-                $old_idx = $idx;
-            }
-        }
-        $idx -= @pre;
-        my $option = $items->[$idx]{name};
-        my $readline_prompt = $items->[$idx]{prompt};
-        $readline_prompt =~ s/\s+\z//;
-        $self->__opt_readline( $ref_section, $option, $readline_prompt );
+        $self->{info}{write_config}++;
     }
 }
 
 
-sub __opt_readline {
-    my ( $self, $ref_section, $option, $prompt ) = @_;
-    my $current = $ref_section->{$option};
-    my $trs = Term::ReadLine::Simple->new();
-    # Readline
-    my $choice = $trs->readline( $prompt . ': ', { default => $current } );
-    return if ! defined $choice;
-    $ref_section->{$option} = $choice;
-    $self->{info}{write_config}++;
-    return;
-}
-
-
 sub __opt_choose_dirs {
-    my ( $self, $ref_section, $option ) = @_;
-    my $current = $ref_section->{$option};
+    my ( $self, $opt_type, $section, $option ) = @_;
+    my $current = $self->{$opt_type}{$section}{$option};
     # Choose_dirs
-    my $dirs = choose_dirs( { mouse => $self->{opt}{G}{mouse}, current => $current } );
+    my $dirs = choose_dirs( { mouse => $self->{opt}{table}{mouse}, current => $current } );
     return if ! defined $dirs;
     return if ! @$dirs;
-    $ref_section->{$option} = $dirs;
+    $self->{$opt_type}{$section}{$option} = $dirs;
     $self->{info}{write_config}++;
     return;
 }
 
 
-sub database_setting {
+sub __database_setting {
     my ( $self, $db ) = @_;
     my $changed = 0;
     SECTION: while ( 1 ) {
@@ -553,7 +535,7 @@ sub database_setting {
                 # Choose
                 $db_plugin = choose(
                     [ undef, map( "- $_", @{$self->{opt}{G}{db_plugins}} ) ],
-                    { %{$self->{info}{lyt_3}}, undef => $self->{info}{conf_back} }
+                    { %{$self->{info}{lyt_3}}, undef => $self->{info}{back_short} }
                 );
                 return if ! defined $db_plugin;
             }
@@ -563,30 +545,39 @@ sub database_setting {
         }
         my $obj_db = App::DBBrowser::DB->new( $self->{info}, $self->{opt} );
         $db_driver = $obj_db->db_driver() if ! $db_driver;
-        my $login_data = $obj_db->login_data();
-        my $connect_attr = $obj_db->connect_attributes();
+        my $env_variables = $obj_db->environment_variables();
+        my $login_data    = $obj_db->read_arguments();
+        my $connect_attr  = $obj_db->choose_arguments();
         my $items = {
-            login_mode   => [ map {
-                { name         => 'login_mode_' . $_->{name},
-                  prompt       => $_->{prompt},
-                  avail_values => [ 'Ask', 'Use DBI_' . uc $_->{name}, 'Don\'t set' ] }
-             } @$login_data ],
-            login_data   => [ grep { ! $_->{keep_secret} } @$login_data ],
-            connect_attr => $connect_attr,
+            required => [ map { {
+                    name         => 'field_' . $_->{name},
+                    prompt       => exists $_->{prompt} ? $_->{prompt} : $_->{name},
+                    avail_values => [ 'NO', 'YES' ]
+                } } @$login_data ],
+            env_variables => [ map { {
+                    name         => $_,
+                    prompt       => $_,
+                    avail_values => [ 'NO', 'YES' ]
+                } } @$env_variables ],
+            read_argument   => [
+                    grep { ! $_->{keep_secret} } @$login_data
+                ],
+            choose_argument => $connect_attr,
         };
-        push @{$items->{connect_attr}}, {
+        push @{$items->{choose_argument}}, {
             name          => 'binary_filter',
             avail_values  => [ 0, 1 ],
             default_index => 0,
         };
         my @groups;
-        push @groups, [ 'login_mode',   "- Login Mode"         ] if @{$items->{login_mode}};
-        push @groups, [ 'login_data',   "- Login Data"         ] if @{$items->{login_data}};
-        push @groups, [ 'connect_attr', "- DB Options"         ];
-        push @groups, [ 'sqlite_dir',   "- Sqlite directories" ] if $db_driver eq 'SQLite';
+        push @groups, [ 'required',        "- Fields"             ] if @{$items->{required}};
+        push @groups, [ 'env_variables',   "- ENV Variables"      ] if @{$items->{env_variables}};
+        push @groups, [ 'read_argument',   "- Login Data"         ] if @{$items->{read_argument}};
+        push @groups, [ 'choose_argument', "- DB Options"         ];
+        push @groups, [ 'sqlite_dir',      "- Sqlite directories" ] if $db_driver eq 'SQLite';
         my $prompt = defined $db ? 'DB: "' . ( $db_driver eq 'SQLite' ? basename $db : $db )
-                                 : 'Plugin: ' . $db_plugin;
-        my $ref_section = $self->{db_opt}{$section};
+                                 : 'Plugin "' . $db_plugin . '"';
+        my $opt_type = 'db_opt';
         my $old_idx_group = 0;
 
         GROUP: while ( 1 ) {
@@ -598,9 +589,9 @@ sub database_setting {
             my $idx_group = choose(
                 $choices,
                 { %{$self->{info}{lyt_3}}, prompt => $prompt, index => 1,
-                  default => $old_idx_group, undef => $self->{info}{conf_back} }
+                  default => $old_idx_group, undef => $self->{info}{back_short} }
             );
-            if ( ! $idx_group ) {
+            if ( ! defined $idx_group || ! defined $choices->[$idx_group] ) {
                 if ( $self->{info}{write_config} ) {
                     $self->__write_db_config_files();
                     delete $self->{info}{write_config};
@@ -645,36 +636,76 @@ sub database_setting {
                 next GROUP;;
             }
             my $group  = $groups[$idx_group-@pre][0];
-            if ( $group eq 'login_mode' ) {
+            if ( $group eq 'required' ) {
                 my $sub_menu = [];
                 for my $item ( @{$items->{$group}} ) {
-                    my $login_mode_key = $item->{name};
-                    push @$sub_menu, [ $login_mode_key, '- ' . $item->{prompt}, $item->{avail_values} ];
-                    if ( ! defined $ref_section->{$login_mode_key} ) {
-                        $ref_section->{$login_mode_key} = 0;
+                    my $required = $item->{name};
+                    push @$sub_menu, [ $required, '- ' . $item->{prompt}, $item->{avail_values} ];
+                    if ( ! defined $self->{db_opt}{$section}{$required} ) {
+                        if ( defined $self->{db_opt}{$db_plugin}{$required} ) {
+                            $self->{db_opt}{$section}{$required} = $self->{db_opt}{$db_plugin}{$required};
+                        }
+                        else {
+                            $self->{db_opt}{$section}{$required} = 1;  # All fields required by default
+                        }
                     }
                 }
-                $self->__opt_choose_multi( $ref_section, $sub_menu, $prompt );
+                my $prompt = 'Required fields (' . $db_plugin . '):';
+                $self->__opt_choose_multi( $opt_type, $section, $sub_menu, $prompt );
                 next GROUP;
             }
-            elsif ( $group eq 'login_data' ) {
-                $self->__group_readline( $ref_section, $items->{$group}, $prompt );
+            elsif ( $group eq 'env_variables' ) {
+                my $sub_menu = [];
+                for my $item ( @{$items->{$group}} ) {
+                    my $env_variable = $item->{name};
+                    push @$sub_menu, [ $env_variable, '- ' . $item->{prompt}, $item->{avail_values} ];
+                    if ( ! defined $self->{db_opt}{$section}{$env_variable} ) {
+                        if ( defined $self->{db_opt}{$db_plugin}{$env_variable} ) {
+                            $self->{db_opt}{$section}{$env_variable} = $self->{db_opt}{$db_plugin}{$env_variable};
+                        }
+                        else {
+                            $self->{db_opt}{$section}{$env_variable} = 0;
+                        }
+                    }
+                }
+                my $prompt = 'Use ENV variables (' . $db_plugin . '):';
+                $self->__opt_choose_multi( $opt_type, $section, $sub_menu, $prompt );
+                next GROUP;
             }
-            elsif ( $group eq 'connect_attr' ) {
+            elsif ( $group eq 'read_argument' ) {
+               for my $item ( @{$items->{$group}} ) {
+                    my $option = $item->{name};
+                    if ( ! defined $self->{db_opt}{$section}{$option} ) {
+                        if ( defined $self->{db_opt}{$db_plugin}{$option} ) {
+                            $self->{db_opt}{$section}{$option} = $self->{db_opt}{$db_plugin}{$option};
+                        }
+                    }
+                }
+                my $prompt = 'Default login data (' . $db_plugin . '):';
+                $self->__group_readline( $opt_type, $section, $items->{$group}, $prompt );
+            }
+            elsif ( $group eq 'choose_argument' ) {
                 my $sub_menu = [];
                 for my $item ( @{$items->{$group}} ) {
                     my $option = $item->{name};
-                    push @$sub_menu, [ $option, '- ' . $option, $item->{avail_values} ];
-                    if ( ! defined $ref_section->{$option} ) {
-                        $ref_section->{$option} = $item->{avail_values}[$item->{default_index}];
+                    my $prompt = '- ' . ( exists $item->{prompt} ? $item->{prompt} : $item->{name} );
+                    push @$sub_menu, [ $option, $prompt, $item->{avail_values} ];
+                    if ( ! defined $self->{db_opt}{$section}{$option} ) {
+                        if ( defined $self->{db_opt}{$db_plugin}{$option} ) {
+                            $self->{db_opt}{$section}{$option} = $self->{db_opt}{$db_plugin}{$option};
+                        }
+                        else {
+                            $self->{db_opt}{$section}{$option} = $item->{avail_values}[$item->{default_index}];
+                        }
                     }
                 }
-                $self->__opt_choose_multi( $ref_section, $sub_menu, $prompt );
+                my $prompt = 'Options (' . $db_plugin . '):';
+                $self->__opt_choose_multi( $opt_type, $section, $sub_menu, $prompt );
                 next GROUP;
             }
             elsif ( $group eq 'sqlite_dir' ) {
                 my $option = 'directories_sqlite';
-                $self->__opt_choose_dirs( $ref_section, $option );
+                $self->__opt_choose_dirs( $opt_type, $section, $option );
                 next GROUP;
             }
         }
@@ -693,7 +724,7 @@ sub __write_config_files {
     }
     my $auxil = App::DBBrowser::Auxil->new( $self->{info} );
     my $file_name = $self->{info}{config_generic};
-    $auxil->write_json( sprintf( $fmt, $file_name ), $tmp  );
+    $auxil->__write_json( sprintf( $fmt, $file_name ), $tmp  );
 }
 
 
@@ -714,19 +745,19 @@ sub __write_db_config_files {
     my $auxil = App::DBBrowser::Auxil->new( $self->{info} );
     for my $section ( keys %$tmp ) {
         my $file_name =  $section;
-        $auxil->write_json( sprintf( $fmt, $file_name ), $tmp->{$section}  );
+        $auxil->__write_json( sprintf( $fmt, $file_name ), $tmp->{$section}  );
     }
 }
 
 
-sub read_config_files {
+sub __read_config_files {
     my ( $self ) = @_;
     $self->{opt} = $self->defaults();
     my $fmt = $self->{info}{conf_file_fmt};
     my $auxil = App::DBBrowser::Auxil->new( $self->{info} );
     my $file =  sprintf( $fmt, $self->{info}{config_generic} );
     if ( -f $file && -s $file ) {
-        my $tmp = $auxil->read_json( $file );
+        my $tmp = $auxil->__read_json( $file );
         for my $section ( keys %$tmp ) {
             for my $option ( keys %{$tmp->{$section}} ) {
                 $self->{opt}{$section}{$option} = $tmp->{$section}{$option} if exists $self->{opt}{$section}{$option};
@@ -737,14 +768,14 @@ sub read_config_files {
 }
 
 
-sub read_db_config_files {
+sub __read_db_config_files {
     my ( $self ) = @_;
     my $fmt = $self->{info}{conf_file_fmt};
     my $auxil = App::DBBrowser::Auxil->new( $self->{info} );
     for my $db_plugin ( @{$self->{opt}{G}{db_plugins}} ) {
         my $file = sprintf( $fmt, $db_plugin );
         if ( -f $file && -s $file ) {
-            my $tmp = $auxil->read_json( $file );
+            my $tmp = $auxil->__read_json( $file );
             for my $conf_sect ( keys %$tmp ) {
                 my $section = $db_plugin . ( $conf_sect =~ /^\*\Q$db_plugin\E\z/ ? '' : '_' . $conf_sect );
                 for my $option ( keys %{$tmp->{$conf_sect}} ) {
@@ -759,18 +790,6 @@ sub read_db_config_files {
 
 
 
-
-
-
-
-
-
-
-
-
-
-
-
 1;
 
 
